@@ -1,4 +1,4 @@
-/*      $Id: client.h,v 1.129 2004/12/30 22:08:37 olivier Exp $
+/*      $Id: client.h 20323 2006-03-10 23:20:58Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -68,85 +68,97 @@
 #define INCLUDE_SKIP_TASKBAR            (1<<4)
 
 #define NO_UPDATE_FLAG                  0
-#define UPDATE_KEYGRABS                 (1<<0)
-#define UPDATE_FRAME                    (1<<1)
-#define UPDATE_GRAVITY                  (1<<2)
-#define UPDATE_CACHE                    (1<<3)
-#define UPDATE_ALL                      (UPDATE_KEYGRABS | \
+#define UPDATE_KEY_GRABS                (1<<0)
+#define UPDATE_BUTTON_GRABS             (1<<1)
+#define UPDATE_FRAME                    (1<<2)
+#define UPDATE_GRAVITY                  (1<<3)
+#define UPDATE_CACHE                    (1<<4)
+#define UPDATE_ALL                      (UPDATE_KEY_GRABS | \
+                                         UPDATE_BUTTON_GRABS | \
                                          UPDATE_FRAME | \
                                          UPDATE_GRAVITY | \
                                          UPDATE_CACHE)
 
-#define CLIENT_MIN_VISIBLE              10      /* pixels */
+#ifndef CLIENT_MIN_VISIBLE
+#define CLIENT_MIN_VISIBLE              15
+#endif
 
-#define XFWM_FLAG_HAS_BORDER           (1L<<0)
-#define XFWM_FLAG_HAS_MENU             (1L<<1)
-#define XFWM_FLAG_HAS_MAXIMIZE         (1L<<2)
-#define XFWM_FLAG_HAS_CLOSE            (1L<<3)
-#define XFWM_FLAG_HAS_HIDE             (1L<<4)
-#define XFWM_FLAG_HAS_MOVE             (1L<<5)
-#define XFWM_FLAG_HAS_RESIZE           (1L<<6)
-#define XFWM_FLAG_HAS_STICK            (1L<<7)
-#define XFWM_FLAG_FOCUS                (1L<<8)
-#define XFWM_FLAG_IS_RESIZABLE         (1L<<9)
-#define XFWM_FLAG_MAP_PENDING          (1L<<10)
-#define XFWM_FLAG_VISIBLE              (1L<<11)
-#define XFWM_FLAG_MANAGED              (1L<<13)
-#define XFWM_FLAG_SESSION_MANAGED      (1L<<14)
-#define XFWM_FLAG_WORKSPACE_SET        (1L<<15)
-#define XFWM_FLAG_WAS_SHOWN            (1L<<16)
-#define XFWM_FLAG_DRAW_ACTIVE          (1L<<17)
-#define XFWM_FLAG_SEEN_ACTIVE          (1L<<18)
-#define XFWM_FLAG_FIRST_MAP            (1L<<19)
-#define XFWM_FLAG_LEGACY_FULLSCREEN    (1L<<20)
-#define XFWM_FLAG_MOVING_RESIZING      (1L<<21)
+#define XFWM_FLAG_HAS_BORDER            (1L<<0)
+#define XFWM_FLAG_HAS_MENU              (1L<<1)
+#define XFWM_FLAG_HAS_MAXIMIZE          (1L<<2)
+#define XFWM_FLAG_HAS_CLOSE             (1L<<3)
+#define XFWM_FLAG_HAS_HIDE              (1L<<4)
+#define XFWM_FLAG_HAS_MOVE              (1L<<5)
+#define XFWM_FLAG_HAS_RESIZE            (1L<<6)
+#define XFWM_FLAG_HAS_STICK             (1L<<7)
+#define XFWM_FLAG_FOCUS                 (1L<<8)
+#define XFWM_FLAG_IS_RESIZABLE          (1L<<9)
+#define XFWM_FLAG_MAP_PENDING           (1L<<10)
+#define XFWM_FLAG_VISIBLE               (1L<<11)
+#define XFWM_FLAG_MANAGED               (1L<<13)
+#define XFWM_FLAG_SESSION_MANAGED       (1L<<14)
+#define XFWM_FLAG_WORKSPACE_SET         (1L<<15)
+#define XFWM_FLAG_WAS_SHOWN             (1L<<16)
+#define XFWM_FLAG_DRAW_ACTIVE           (1L<<17)
+#define XFWM_FLAG_SEEN_ACTIVE           (1L<<18)
+#define XFWM_FLAG_FIRST_MAP             (1L<<19)
+#define XFWM_FLAG_LEGACY_FULLSCREEN     (1L<<20)
+#define XFWM_FLAG_MOVING_RESIZING       (1L<<21)
 
-#define CLIENT_FLAG_HAS_STRUT          (1L<<0)
-#define CLIENT_FLAG_HAS_STRUT_PARTIAL  (1L<<1)
-#define CLIENT_FLAG_HAS_USER_TIME      (1L<<2)
-#define CLIENT_FLAG_ABOVE              (1L<<3)
-#define CLIENT_FLAG_BELOW              (1L<<4)
-#define CLIENT_FLAG_FULLSCREEN         (1L<<5)
-#define CLIENT_FLAG_ICONIFIED          (1L<<6)
-#define CLIENT_FLAG_MAXIMIZED_VERT     (1L<<7)
-#define CLIENT_FLAG_MAXIMIZED_HORIZ    (1L<<8)
-#define CLIENT_FLAG_MAXIMIZED          (CLIENT_FLAG_MAXIMIZED_VERT | \
-                                        CLIENT_FLAG_MAXIMIZED_HORIZ)
-#define CLIENT_FLAG_SHADED             (1L<<9)
-#define CLIENT_FLAG_SKIP_PAGER         (1L<<10)
-#define CLIENT_FLAG_SKIP_TASKBAR       (1L<<11)
-#define CLIENT_FLAG_STATE_MODAL        (1L<<12)
-#define CLIENT_FLAG_STICKY             (1L<<13)
-#define CLIENT_FLAG_NAME_CHANGED       (1L<<15)
-#define CLIENT_FLAG_DEMANDS_ATTENTION  (1L<<16)
+#define CLIENT_FLAG_HAS_STRUT           (1L<<0)
+#define CLIENT_FLAG_HAS_STRUT_PARTIAL   (1L<<1)
+#define CLIENT_FLAG_HAS_USER_TIME       (1L<<2)
+#define CLIENT_FLAG_HAS_STARTUP_TIME    (1L<<3)
+#define CLIENT_FLAG_ABOVE               (1L<<4)
+#define CLIENT_FLAG_BELOW               (1L<<5)
+#define CLIENT_FLAG_FULLSCREEN          (1L<<6)
+#define CLIENT_FLAG_ICONIFIED           (1L<<7)
+#define CLIENT_FLAG_MAXIMIZED_VERT      (1L<<8)
+#define CLIENT_FLAG_MAXIMIZED_HORIZ     (1L<<9)
+#define CLIENT_FLAG_MAXIMIZED           (CLIENT_FLAG_MAXIMIZED_VERT | \
+                                         CLIENT_FLAG_MAXIMIZED_HORIZ)
+#define CLIENT_FLAG_SHADED              (1L<<10)
+#define CLIENT_FLAG_SKIP_PAGER          (1L<<11)
+#define CLIENT_FLAG_SKIP_TASKBAR        (1L<<12)
+#define CLIENT_FLAG_STATE_MODAL         (1L<<13)
+#define CLIENT_FLAG_STICKY              (1L<<15)
+#define CLIENT_FLAG_NAME_CHANGED        (1L<<16)
+#define CLIENT_FLAG_DEMANDS_ATTENTION   (1L<<17)
+#define CLIENT_FLAG_HAS_SHAPE           (1L<<18)
 
-#define WM_FLAG_DELETE                 (1L<<0)
-#define WM_FLAG_INPUT                  (1L<<1)
-#define WM_FLAG_TAKEFOCUS              (1L<<2)
-#define WM_FLAG_CONTEXT_HELP           (1L<<3)
-#define WM_FLAG_URGENT                 (1L<<4)
+#define WM_FLAG_DELETE                  (1L<<0)
+#define WM_FLAG_INPUT                   (1L<<1)
+#define WM_FLAG_TAKEFOCUS               (1L<<2)
+#define WM_FLAG_CONTEXT_HELP            (1L<<3)
+#define WM_FLAG_URGENT                  (1L<<4)
 
-#define CLIENT_FLAG_INITIAL_VALUES     XFWM_FLAG_HAS_BORDER | \
-                                       XFWM_FLAG_HAS_MENU | \
-                                       XFWM_FLAG_HAS_MAXIMIZE | \
-                                       XFWM_FLAG_HAS_STICK | \
-                                       XFWM_FLAG_HAS_HIDE | \
-                                       XFWM_FLAG_HAS_CLOSE | \
-                                       XFWM_FLAG_HAS_MOVE | \
-                                       XFWM_FLAG_HAS_RESIZE | \
-                                       XFWM_FLAG_FIRST_MAP
+#define CLIENT_FLAG_INITIAL_VALUES      XFWM_FLAG_HAS_BORDER | \
+                                        XFWM_FLAG_HAS_MENU | \
+                                        XFWM_FLAG_HAS_MAXIMIZE | \
+                                        XFWM_FLAG_HAS_STICK | \
+                                        XFWM_FLAG_HAS_HIDE | \
+                                        XFWM_FLAG_HAS_CLOSE | \
+                                        XFWM_FLAG_HAS_MOVE | \
+                                        XFWM_FLAG_HAS_RESIZE | \
+                                        XFWM_FLAG_FIRST_MAP
 
-#define ALL_WORKSPACES                 (int) 0xFFFFFFFF
+#define ALL_WORKSPACES                  (int) 0xFFFFFFFF
 
-#define CONSTRAINED_WINDOW(c)          ((c->win_layer > WIN_LAYER_DESKTOP) && \
-                                        (c->win_layer < WIN_LAYER_ABOVE_DOCK) && \
-                                       !(c->type & (WINDOW_DESKTOP | WINDOW_DOCK)) && \
-                                       !FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
+#define CONSTRAINED_WINDOW(c)           ((c->win_layer > WIN_LAYER_DESKTOP) && \
+                                         (c->win_layer < WIN_LAYER_ABOVE_DOCK) && \
+                                        !(c->type & (WINDOW_DESKTOP | WINDOW_DOCK)) && \
+                                        !FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN))
 
-#define WINDOW_TYPE_DIALOG             (WINDOW_DIALOG | WINDOW_MODAL_DIALOG)
-#define WINDOW_TYPE_DONT_PLACE         (WINDOW_DESKTOP | WINDOW_DOCK | WINDOW_SPLASHSCREEN)
-#define WINDOW_REGULAR_FOCUSABLE       (WINDOW_NORMAL | WINDOW_UTILITY | WINDOW_DIALOG | WINDOW_MODAL_DIALOG)
-#define WINDOW_TYPE_DONT_FOCUS         (WINDOW_DOCK)
+#define WINDOW_TYPE_DIALOG              (WINDOW_DIALOG | \
+                                         WINDOW_MODAL_DIALOG)
+#define WINDOW_TYPE_DONT_PLACE          (WINDOW_DESKTOP | \
+                                         WINDOW_DOCK | \
+                                         WINDOW_UTILITY | \
+                                         WINDOW_SPLASHSCREEN)
+#define WINDOW_REGULAR_FOCUSABLE        (WINDOW_NORMAL | \
+                                         WINDOW_DIALOG | \
+                                         WINDOW_MODAL_DIALOG)
+#define WINDOW_TYPE_DONT_FOCUS          (WINDOW_DOCK)
 
 
 
@@ -185,8 +197,8 @@ netWindowType;
 
 struct _ClientPixmapCache
 {
-    XfwmPixmap pm_title[2];
-    XfwmPixmap pm_sides[3][2];
+    xfwmPixmap pm_title[2];
+    xfwmPixmap pm_sides[3][2];
     int previous_width;
     int previous_height;
 };
@@ -206,6 +218,7 @@ struct _Client
     xfwmWindow buttons[BUTTON_COUNT];
     Window client_leader;
     Window group_leader;
+    xfwmPixmap appmenu[3];
     Colormap cmap;
     unsigned long win_hints;
     unsigned long win_state;
@@ -223,6 +236,7 @@ struct _Client
     int y;
     int width;
     int height;
+    int depth;
     int border_width;
     int gravity;
     int win_workspace;
@@ -252,7 +266,7 @@ struct _Client
     ClientPixmapCache pm_cache;
     /* Opacity for the compositor */
     guint opacity;
-
+    
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     /* Startup notification */
     char *startup_id;
@@ -273,6 +287,7 @@ void     clientGetMWMHints (Client *, gboolean);
 void     clientGetWMNormalHints (Client *, gboolean);
 void     clientGetWMProtocols (Client *);
 void     clientClearPixmapCache (Client *);
+void     clientUpdateIcon (Client * c);
 Client  *clientFrame (DisplayInfo *, Window, gboolean);
 void     clientUnframe (Client *, gboolean);
 void     clientFrameAll (ScreenInfo *);
@@ -288,12 +303,11 @@ Client  *clientGetFromWindow (ScreenInfo *, Window, int);
 void     clientShow (Client *, gboolean);
 void     clientHide (Client *, int, gboolean);
 void     clientHideAll (Client *, int);
-void     clientToggleShowDesktop (ScreenInfo *, gboolean);
+void     clientClearAllShowDesktop (ScreenInfo *);
+void     clientToggleShowDesktop (ScreenInfo *);
 void     clientClose (Client *);
 void     clientKill (Client *);
 void     clientEnterContextMenuState (Client *);
-void     clientRaise (Client *);
-void     clientLower (Client *);
 void     clientSetLayer (Client *, int);
 void     clientSetWorkspace (Client *, int, gboolean);
 void     clientShade (Client *);
@@ -306,7 +320,9 @@ void     clientToggleFullscreen (Client *);
 void     clientToggleAbove (Client *);
 void     clientToggleBelow (Client *);
 void     clientRemoveMaximizeFlag (Client *);
-void     clientToggleMaximized (Client *, int);
+void     clientToggleMaximized (Client *, int, gboolean);
+void     clientIncOpacity (Client *);
+void     clientDecOpacity (Client *);
 void     clientScreenResize(ScreenInfo *);
 void     clientMove (Client *, XEvent *);
 void     clientResize (Client *, int, XEvent *);
@@ -315,6 +331,7 @@ void     clientButtonPress (Client *, Window, XButtonEvent *);
 Client  *clientGetLeader (Client *);
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 char    *clientGetStartupId (Client *);
-#endif
+#endif /* HAVE_LIBSTARTUP_NOTIFICATION */
+
 
 #endif /* INC_CLIENT_H */
