@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 12328 2005-01-30 13:24:07Z olivier $
+/*      $Id: compositor.c 20893 2006-04-14 20:53:25Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
         xcompmgr - (c) 2003 Keith Packard
         metacity - (c) 2003, 2004 Red Hat, Inc.
-        xfwm4    - (c) 2004 Olivier Fourdan
+        xfwm4    - (c) 2005-2006 Olivier Fourdan
 
 */
 
@@ -31,6 +31,7 @@
 
 #include <glib.h>
 #include <math.h>
+#include <string.h>
 #include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
@@ -46,10 +47,6 @@
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xrender.h>
 
-#define WINDOW_SOLID    0
-#define WINDOW_TRANS    1
-#define WINDOW_ARGB     2
-
 #ifndef SHADOW_RADIUS
 #define SHADOW_RADIUS   6
 #endif /* SHADOW_RADIUS */
@@ -66,6 +63,8 @@
 #define SHADOW_OFFSET_Y (SHADOW_RADIUS * -5 / 4)
 #endif /* SHADOW_OFFSET_Y */
 
+#define IDLE_REPAINT
+
 typedef struct _CWindow CWindow;
 struct _CWindow
 {
@@ -77,6 +76,7 @@ struct _CWindow
     Damage damage;
     gboolean damaged;
     gboolean viewable;
+    gboolean shaped;
 
 #if HAVE_NAME_WINDOW_PIXMAP
     Pixmap name_window_pixmap;
@@ -85,6 +85,7 @@ struct _CWindow
     Picture shadow;
     Picture alphaPict;
     Picture shadowPict;
+    Picture alphaBorderPict;
     XserverRegion borderSize;
     XserverRegion borderClip;
     XserverRegion extents;
@@ -92,8 +93,8 @@ struct _CWindow
     gint shadow_dy;
     gint shadow_width;
     gint shadow_height;
-    gint mode;
     guint opacity;
+    gboolean argb;
     gboolean skipped;
 };
 
@@ -138,6 +139,24 @@ find_cwindow_in_display (DisplayInfo *display_info, Window id)
     return NULL;
 }
 
+static gboolean
+is_shaped (DisplayInfo *display_info, Window id)
+{
+    int xws, yws, xbs, ybs;
+    unsigned wws, hws, wbs, hbs;
+    int boundingShaped, clipShaped;
+
+    g_return_val_if_fail (display_info != NULL, FALSE);
+
+    if (display_info->have_shape)
+    {
+        XShapeQueryExtents (display_info->dpy, id, &boundingShaped, &xws, &yws, &wws,
+                            &hws, &clipShaped, &xbs, &ybs, &wbs, &hbs);
+        return (boundingShaped != 0);
+    }
+    return FALSE;
+}
+
 static gdouble
 gaussian (gdouble r, gdouble x, gdouble y)
 {
@@ -490,6 +509,7 @@ shadow_picture (ScreenInfo *screen_info, gdouble opacity,
     if (shadowImage == NULL)
     {
         *wp = *hp = 0;
+        g_warning ("(shadowImage != NULL) failed");
         return (None);
     }
 
@@ -503,7 +523,7 @@ shadow_picture (ScreenInfo *screen_info, gdouble opacity,
     }
 
     shadowPicture = XRenderCreatePicture (myScreenGetXDisplay (screen_info),
-                                        shadowPixmap, render_format, 0, 0);
+                                        shadowPixmap, render_format, 0, NULL);
     if (shadowPicture == None)
     {
         XDestroyImage (shadowImage);
@@ -512,7 +532,7 @@ shadow_picture (ScreenInfo *screen_info, gdouble opacity,
         return None;
     }
 
-    gc = XCreateGC (myScreenGetXDisplay (screen_info), shadowPixmap, 0, 0);
+    gc = XCreateGC (myScreenGetXDisplay (screen_info), shadowPixmap, 0, NULL);
     if (gc == None)
     {
         XDestroyImage (shadowImage);
@@ -558,7 +578,7 @@ solid_picture (ScreenInfo *screen_info, gboolean argb,
 
     pa.repeat = TRUE;
     picture = XRenderCreatePicture (myScreenGetXDisplay (screen_info), pixmap,
-                                    render_format, CPRepeat,  &pa);
+                                    render_format, CPRepeat, &pa);
     if (picture == None)
     {
         XFreePixmap (myScreenGetXDisplay (screen_info), pixmap);
@@ -667,10 +687,10 @@ root_tile (ScreenInfo *screen_info)
     {
         XRenderColor c;
 
-        /* Just a color by default #6985b7 - Same as xfdesktop */
-        c.red   = 0x6900;
-        c.green = 0x8500;
-        c.blue  = 0xB700;
+        /* Just a color by default #6A7065 */
+        c.red   = 0x7f00;
+        c.green = 0x7f00;
+        c.blue  = 0x7f00;
         c.alpha = 0xffff;
         XRenderFillRectangle (dpy, PictOpSrc, picture, &c, 0, 0, 1, 1);
     }
@@ -682,7 +702,7 @@ create_root_buffer (ScreenInfo *screen_info)
 {
     Picture pict;
     XRenderPictFormat *format;
-    Pixmap  rootPixmap;
+    Pixmap rootPixmap;
     Visual *visual;
     gint depth;
     gint screen_width;
@@ -739,6 +759,10 @@ win_extents (CWindow *cw)
     ScreenInfo *screen_info = NULL;
     Client *c;
     XRectangle r;
+    gboolean has_frame;
+    gboolean is_shaped;
+    gboolean is_argb;
+    gboolean is_override;
 
     g_return_val_if_fail (cw != NULL, None);
     TRACE ("entering win_extents: 0x%lx", cw->id);
@@ -750,54 +774,73 @@ win_extents (CWindow *cw)
     r.width = cw->attr.width + cw->attr.border_width * 2;
     r.height = cw->attr.height + cw->attr.border_width * 2;
 
-    if (c)
+    /* 
+       We apply a shadow to the window if:
+       
+       - It's a window with a frame and the user asked for shadows under regular
+         windows,
+       - it's an override redirect window that is not shaped, not an argb and
+         the user asked for shadows on so called "popup" windows.
+     */
+
+    is_override = (c == NULL);
+    is_argb = (cw->argb);
+    has_frame = (!is_override && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER));
+    is_shaped = ((!is_override && FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
+                 || (is_override && (cw->shaped)));
+
+    if ((is_override  && !(is_argb || is_shaped) && screen_info->params->show_popup_shadow) || 
+       ((!is_override && (has_frame || !(is_argb || is_shaped)) && screen_info->params->show_frame_shadow)))
     {
-        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER) && (cw->mode != WINDOW_ARGB))
+        XRectangle sr;
+
+        TRACE ("window 0x%lx has extents", cw->id);
+        cw->shadow_dx = SHADOW_OFFSET_X + screen_info->params->shadow_delta_x;
+        cw->shadow_dy = SHADOW_OFFSET_Y + screen_info->params->shadow_delta_y;
+
+        if (!(cw->shadow))
         {
-            XRectangle sr;
+            double shadow_opacity;
+            shadow_opacity = (double) screen_info->params->frame_opacity
+                                      * SHADOW_OPACITY
+                                      * cw->opacity
+                                      / (NET_WM_OPAQUE * 100.0);
 
-            TRACE ("window 0x%lx (%s) has extents", cw->id, c->name);
-            cw->shadow_dx = SHADOW_OFFSET_X + screen_info->params->shadow_delta_x;
-            cw->shadow_dy = SHADOW_OFFSET_Y + screen_info->params->shadow_delta_y;
-            if (!(cw->shadow))
-            {
-                double opacity = SHADOW_OPACITY;
-                if (cw->mode == WINDOW_TRANS)
-                {
-                    opacity = opacity * ((double) cw->opacity) / ((double) NET_WM_OPAQUE);
-                }
-                cw->shadow = shadow_picture (screen_info, opacity,
-                                             cw->attr.width + 2 * cw->attr.border_width,
-                                             cw->attr.height + 2 * cw->attr.border_width,
-                                             &cw->shadow_width, &cw->shadow_height);
-            }
+            cw->shadow = shadow_picture (screen_info, shadow_opacity,
+                                         cw->attr.width + 2 * cw->attr.border_width,
+                                         cw->attr.height + 2 * cw->attr.border_width,
+                                         &cw->shadow_width, &cw->shadow_height);
+        }
 
-            sr.x = cw->attr.x + cw->shadow_dx;
-            sr.y = cw->attr.y + cw->shadow_dy;
-            sr.width = cw->shadow_width;
-            sr.height = cw->shadow_height;
+        sr.x = cw->attr.x + cw->shadow_dx;
+        sr.y = cw->attr.y + cw->shadow_dy;
+        sr.width = cw->shadow_width;
+        sr.height = cw->shadow_height;
 
-            if (sr.x < r.x)
-            {
-                r.width = (r.x + r.width) - sr.x;
-                r.x = sr.x;
-            }
-            if (sr.y < r.y)
-            {
-                r.height = (r.y + r.height) - sr.y;
-                r.y = sr.y;
-            }
-            if (sr.x + sr.width > r.x + r.width)
-            {
-                r.width = sr.x + sr.width - r.x;
-            }
-            if (sr.y + sr.height > r.y + r.height)
-            {
-                r.height = sr.y + sr.height - r.y;
-            }
+        if (sr.x < r.x)
+        {
+            r.width = (r.x + r.width) - sr.x;
+            r.x = sr.x;
+        }
+        if (sr.y < r.y)
+        {
+            r.height = (r.y + r.height) - sr.y;
+            r.y = sr.y;
+        }
+        if (sr.x + sr.width > r.x + r.width)
+        {
+            r.width = sr.x + sr.width - r.x;
+        }
+        if (sr.y + sr.height > r.y + r.height)
+        {
+            r.height = sr.y + sr.height - r.y;
         }
     }
-
+    else if (cw->shadow)
+    {
+        XRenderFreePicture (myScreenGetXDisplay (cw->screen_info), cw->shadow);
+        cw->shadow = None;
+    }
     return XFixesCreateRegion (myScreenGetXDisplay (screen_info), &r, 1);
 }
 
@@ -875,6 +918,134 @@ get_window_picture (CWindow *cw)
     return None;
 }
 
+static void
+paint_win (CWindow *cw, XserverRegion region, gboolean solid_part)
+{
+    Client *c;
+    ScreenInfo *screen_info = NULL;
+    DisplayInfo *display_info = NULL;
+    gboolean has_frame;
+    gboolean paint_solid;
+    
+    g_return_if_fail (cw != NULL);
+    TRACE ("entering paint_win: 0x%lx", cw->id);
+
+    screen_info = cw->screen_info;
+    display_info = screen_info->display_info;
+    c = cw->c;
+    has_frame = (c && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER));
+    paint_solid = ((solid_part) && !(cw->argb) && (cw->opacity == NET_WM_OPAQUE));
+    
+    if ((has_frame) && (screen_info->params->frame_opacity < 100))
+    {
+        int frame_x, frame_y, frame_width, frame_height;
+        int frame_top, frame_bottom, frame_left, frame_right;
+
+        frame_x = frameX (c);
+        frame_y = frameY (c);
+        frame_width = frameWidth (c);
+        frame_height = frameHeight (c);
+        frame_top = frameTop (c);
+        frame_bottom = frameBottom (c);
+        frame_left = frameLeft (c);
+        frame_right = frameRight (c);
+
+        if (!solid_part)
+        {
+            if (!cw->alphaBorderPict)
+            {
+                double frame_opacity;
+                frame_opacity = (double) cw->opacity
+                                         * screen_info->params->frame_opacity
+                                         / (NET_WM_OPAQUE * 100.0);
+
+                cw->alphaBorderPict = solid_picture (screen_info, FALSE, frame_opacity, 0, 0, 0);
+            }
+
+            /* Top Border (title bar) */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              0, 0, 
+                              0, 0, 
+                              frame_x, frame_y, 
+                              frame_width, frame_top);
+
+            /* Bottom Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              0, frame_height - frame_bottom, 
+                              0, 0,
+                              frame_x, frame_y + frame_height - frame_bottom, 
+                              frame_width, frame_bottom);
+            /* Left Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              0, frame_top, 
+                              0, 0,
+                              frame_x, frame_y + frame_top, 
+                              frame_left, frame_height - frame_top - frame_bottom);
+
+            /* Right Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              frame_width - frame_right, frame_top, 
+                              0, 0,
+                              frame_x + frame_width - frame_right, 
+                              frame_y + frame_top, frame_right, 
+                              frame_height - frame_top - frame_bottom);
+        }
+        /* Client Window */
+        if (paint_solid)
+        {
+            XRectangle  r;
+            XserverRegion client_region;
+
+            XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, region);
+            XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None,
+                              screen_info->rootBuffer, 
+                              frame_left, frame_top, 
+                              0, 0,
+                              frame_x + frame_left, frame_y + frame_top, 
+                              frame_width - frame_left - frame_right, frame_height - frame_top - frame_bottom);
+
+            r.x = frame_x + frame_left;
+            r.y = frame_y + frame_top;
+            r.width = frame_width - frame_left - frame_right;
+            r.height = frame_height - frame_top - frame_bottom;
+            client_region = XFixesCreateRegion (display_info->dpy, &r, 1);
+            XFixesSubtractRegion (display_info->dpy, region, region, client_region);
+            XFixesDestroyRegion (display_info->dpy, client_region);
+        }
+        else if (!solid_part)
+        {
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict,
+                              screen_info->rootBuffer, 
+                              frame_left, frame_top, 
+                              0, 0,
+                              frame_x + frame_left, frame_y + frame_top, 
+                              frame_width - frame_left - frame_right, frame_height - frame_top - frame_bottom);
+        }
+    }
+    else
+    {
+        gint x, y, w, h;
+
+        get_paint_bounds (cw, &x, &y, &w, &h);
+        if (paint_solid)
+        {
+            XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, region);
+            XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None, screen_info->rootBuffer,
+                              0, 0, 0, 0, x, y, w, h);
+            XFixesSubtractRegion (display_info->dpy, region, region, cw->borderSize);
+        }
+        else if (!solid_part)
+        {
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict, screen_info->rootBuffer, 
+                              0, 0, 0, 0, x, y, w, h);
+        }
+    }
+}
+
 static void
 paint_all (ScreenInfo *screen_info, XserverRegion region)
 {
@@ -889,7 +1060,6 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
     TRACE ("entering paint_all");
     g_return_if_fail (screen_info);
 
-
     display_info = screen_info->display_info;
     dpy = display_info->dpy;
     screen_width = gdk_screen_get_width (screen_info->gscr);
@@ -956,14 +1126,9 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
         {
             cw->picture = get_window_picture (cw);
         }
-        if (cw->mode == WINDOW_SOLID)
+        if (!(cw->argb) && (cw->opacity == NET_WM_OPAQUE))
         {
-            gint x, y, w, h;
-            get_paint_bounds (cw, &x, &y, &w, &h);
-            XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, region);
-            XRenderComposite (dpy, PictOpSrc, cw->picture, None, screen_info->rootBuffer,
-                              0, 0, 0, 0, x, y, w, h);
-            XFixesSubtractRegion (dpy, region, region, cw->borderSize);
+            paint_win (cw, region, TRUE);
         }
         if (cw->borderClip == None)
         {
@@ -987,6 +1152,7 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
     for (index = g_list_last(screen_info->cwindows); index; index = g_list_previous (index))
     {
         CWindow *cw = (CWindow *) index->data;
+        XserverRegion shadowClip = None;
 
         TRACE ("painting backward 0x%lx", cw->id);
 
@@ -998,31 +1164,35 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
 
         if (cw->shadow)
         {
-            XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, cw->borderClip);
+            shadowClip = XFixesCreateRegion(dpy, 0, 0);
+            XFixesSubtractRegion (dpy, shadowClip, cw->borderClip, cw->borderSize);
+
+            XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, shadowClip);
             XRenderComposite (dpy, PictOpOver, screen_info->blackPicture, cw->shadow,
                               screen_info->rootBuffer, 0, 0, 0, 0,
                               cw->attr.x + cw->shadow_dx,
                               cw->attr.y + cw->shadow_dy,
                               cw->shadow_width, cw->shadow_height);
         }
-        if (cw->mode != WINDOW_SOLID)
-        {
-            gint x, y, w, h;
 
+        if (cw->picture)
+        {
             if ((cw->opacity != NET_WM_OPAQUE) && !(cw->alphaPict))
             {
                 cw->alphaPict = solid_picture (screen_info, FALSE,
                                                (double) cw->opacity / NET_WM_OPAQUE, 0, 0, 0);
             }
-
             XFixesIntersectRegion (dpy, cw->borderClip, cw->borderClip, cw->borderSize);
             XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, cw->borderClip);
-
-            /* cw->alphaPict can be None in the case of ARGB windows, that's ok */
-            get_paint_bounds (cw, &x, &y, &w, &h);
-            XRenderComposite (dpy, PictOpOver, cw->picture, cw->alphaPict,
-                              screen_info->rootBuffer, 0, 0, 0, 0, x, y, w, h);
+            paint_win (cw, None, FALSE);
         }
+
+        if (shadowClip)
+        {
+            XFixesDestroyRegion (dpy, shadowClip);
+            shadowClip = None;
+        }
+
         if (cw->borderClip)
         {
             XFixesDestroyRegion (dpy, cw->borderClip);
@@ -1115,15 +1285,14 @@ compositor_timeout_cb (gpointer data)
 static void
 add_repair (DisplayInfo *display_info)
 {
+#ifdef IDLE_REPAINT
     if (display_info->compositor_idle_id != 0)
     {
         return;
     }
-
     display_info->compositor_idle_id =
         g_idle_add_full (G_PRIORITY_HIGH_IDLE,
                          compositor_idle_cb, display_info, NULL);
-
     if (display_info->compositor_timeout_id != 0)
     {
         return;
@@ -1132,7 +1301,9 @@ add_repair (DisplayInfo *display_info)
     display_info->compositor_timeout_id = 
         g_timeout_add (50 /* ms */, 
                        compositor_timeout_cb, display_info);
-
+#else
+    repair_display (display_info);
+#endif
 }
 
 static void
@@ -1163,6 +1334,13 @@ free_win_data (CWindow *cw, gboolean delete)
         XRenderFreePicture (myScreenGetXDisplay (cw->screen_info), cw->shadowPict);
         cw->shadowPict = None;
     }
+
+    if (cw->alphaBorderPict)
+    {
+        XRenderFreePicture (myScreenGetXDisplay (cw->screen_info), cw->alphaBorderPict);
+        cw->alphaBorderPict = None;
+    }
+
     if ((delete) && (cw->damage != None))
     {
         XDamageDestroy (myScreenGetXDisplay (cw->screen_info), cw->damage);
@@ -1267,7 +1445,7 @@ damage_win (CWindow *cw)
 {
 #ifdef HAVE_COMPOSITOR
     XserverRegion extents;
-
+    
     g_return_if_fail (cw != NULL);
     TRACE ("entering damage_win");
 
@@ -1295,19 +1473,14 @@ determine_mode(CWindow *cw)
         cw->shadowPict = None;
     }
 
+    if (cw->alphaBorderPict)
+    {
+        XRenderFreePicture (myScreenGetXDisplay (cw->screen_info), cw->alphaBorderPict);
+        cw->alphaBorderPict = None;
+    }
+
     format = XRenderFindVisualFormat (myScreenGetXDisplay (screen_info), cw->attr.visual);
-    if ((format) && (format->type == PictTypeDirect) && (format->direct.alphaMask))
-    {
-        cw->mode = WINDOW_ARGB;
-    }
-    else if (cw->opacity != NET_WM_OPAQUE)
-    {
-        cw->mode = WINDOW_TRANS;
-    }
-    else
-    {
-        cw->mode = WINDOW_SOLID;
-    }
+    cw->argb = ((format) && (format->type == PictTypeDirect) && (format->direct.alphaMask));
 
     if (cw->extents)
     {
@@ -1394,11 +1567,36 @@ unmap_win (CWindow *cw)
 }
 
 static void
-add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
+init_opacity (CWindow *cw)
+{
+    ScreenInfo *screen_info = NULL;
+    DisplayInfo *display_info = NULL;
+    Client *c = NULL;
+
+    TRACE ("set_opacity");
+    g_return_if_fail (cw != NULL);
+    
+    screen_info = cw->screen_info;
+    display_info = screen_info->display_info;    
+    c = cw->c;
+
+    if (c)
+    {
+        cw->opacity = c->opacity;
+    }
+    else
+    {
+        cw->opacity = (double) (screen_info->params->popup_opacity / 100.0) * NET_WM_OPAQUE;
+    }
+    getOpacity (display_info, cw->id, &cw->opacity);
+}
+
+static void
+add_win (DisplayInfo *display_info, Window id, Client *c)
 {
     ScreenInfo *screen_info = NULL;
     CWindow *new;
-
+        
     TRACE ("entering add_win: 0x%lx", id);
 
     new = g_new0 (CWindow, 1);
@@ -1444,7 +1642,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
     }
 
     /* Listen for XShape events if applicable */
-    if (display_info->shape)
+    if (display_info->have_shape)
     {
         XShapeSelectInput (display_info->dpy, id, ShapeNotifyMask);
     }
@@ -1453,6 +1651,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
     new->screen_info = screen_info;
     new->id = id;
     new->damaged = FALSE;
+    new->shaped = is_shaped (display_info, id);
     new->viewable = (new->attr.map_state == IsViewable);
     if ((new->attr.class != InputOnly) && (id != screen_info->xroot))
     {
@@ -1467,6 +1666,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
 #endif
     new->picture = None;
     new->alphaPict = None;
+    new->alphaBorderPict = None;
     new->shadowPict = None;
     new->borderSize = None;
     new->extents = None;
@@ -1476,8 +1676,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
     new->shadow_width = 0;
     new->shadow_height = 0;
     new->borderClip = None;
-    new->opacity = opacity;
-    getOpacity (display_info, id, &new->opacity);
+    init_opacity (new);
     determine_mode (new);
 
     /* Insert window at top of stack */
@@ -1580,7 +1779,6 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
 
     if ((cw->attr.width != width) || (cw->attr.height != height))
     {
-
 #if HAVE_NAME_WINDOW_PIXMAP
         if (cw->name_window_pixmap)
         {
@@ -1794,25 +1992,11 @@ compositorHandleConfigureNotify (DisplayInfo *display_info, XConfigureEvent *ev)
     }
 
     cw = find_cwindow_in_display (display_info, ev->window);
-    if (!cw)
+    if (cw)
     {
-        ScreenInfo *screen_info = myDisplayGetScreenFromWindow (display_info, ev->window);
-
-        if ((screen_info) && (ev->window == screen_info->xroot))
-        {
-            if (screen_info->rootBuffer)
-            {
-                XRenderFreePicture (display_info->dpy, screen_info->rootBuffer);
-                screen_info->rootBuffer = None;
-            }
-            add_repair (display_info);
-        }
-
-        return;
+        restack_win (cw, ev->above);
+        resize_win (cw, ev->x, ev->y, ev->width, ev->height, ev->border_width, FALSE);
     }
-
-    restack_win (cw, ev->above);
-    resize_win (cw, ev->x, ev->y, ev->width, ev->height, ev->border_width, FALSE);
 }
 
 static void
@@ -1916,6 +2100,41 @@ compositorHandleDestroyNotify (DisplayInfo *display_info, XDestroyWindowEvent *e
     compositorRemoveWindow (display_info, ev->window);
 }
 
+static void
+compositorHandleShapeNotify (DisplayInfo *display_info, XShapeEvent *ev)
+{
+    CWindow *cw;
+
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (ev != NULL);
+    TRACE ("entering compositorHandleShapeNotify for 0x%lx", ev->window);
+
+    if (!(display_info->enable_compositor))
+    {
+        TRACE ("compositor disabled");
+        return;
+    }
+    
+    cw = find_cwindow_in_display (display_info, ev->window);
+    if (cw)
+    {
+        if (ev->kind == ShapeBounding)
+        {
+            if ((ev->shaped) && !(cw->shaped))
+            {
+                cw->shaped = TRUE;
+            }
+            resize_win (cw, cw->attr.x, cw->attr.y, 
+                            ev->width + ev->x, ev->height + ev->y, 
+                            cw->attr.border_width, TRUE);
+            if (!(ev->shaped) && (cw->shaped))
+            {
+                cw->shaped = FALSE;
+            }
+        }
+    }
+}
+
 #endif /* HAVE_COMPOSITOR */
 
 void
@@ -2023,10 +2242,7 @@ compositorAddWindow (DisplayInfo *display_info, Window id, Client *c)
     }
     else
     {
-        guint opacity;
-
-        opacity = ((c != NULL) ? c->opacity : NET_WM_OPAQUE);
-        add_win (display_info, id, c, opacity);
+        add_win (display_info, id, c);
     }
 #endif /* HAVE_COMPOSITOR */
 }
@@ -2102,6 +2318,10 @@ compositorHandleEvent (DisplayInfo *display_info, XEvent *ev)
     {
         compositorHandleDamage (display_info, (XDamageNotifyEvent *) ev);
     }
+    else if (ev->type == (display_info->shape_event_base + ShapeNotify))
+    {
+        compositorHandleShapeNotify (display_info, (XShapeEvent *) ev);
+    }
 #endif /* HAVE_COMPOSITOR */
 }
 
@@ -2149,7 +2369,12 @@ compositorInitDisplay (DisplayInfo *display_info)
     else
     {
         display_info->have_composite = TRUE;
+#if DEBUG
+        g_print ("composite event base: %i\n", display_info->composite_event_base);
+        g_print ("composite error base: %i\n", display_info->composite_error_base);
+#endif
     }
+
     if (!XDamageQueryExtension (display_info->dpy,
                             &display_info->damage_event_base,
                             &display_info->damage_error_base))
@@ -2162,7 +2387,12 @@ compositorInitDisplay (DisplayInfo *display_info)
     else
     {
         display_info->have_damage = TRUE;
+#if DEBUG
+        g_print ("damage event base: %i\n", display_info->damage_event_base);
+        g_print ("damage error base: %i\n", display_info->damage_error_base);
+#endif
     }
+
     if (!XFixesQueryExtension (display_info->dpy,
                             &display_info->fixes_event_base,
                             &display_info->fixes_error_base))
@@ -2175,12 +2405,17 @@ compositorInitDisplay (DisplayInfo *display_info)
     else
     {
         display_info->have_fixes = TRUE;
+#if DEBUG
+        g_print ("fixes event base: %i\n", display_info->fixes_event_base);
+        g_print ("fixes error base: %i\n", display_info->fixes_error_base);
+#endif
     }
 
     display_info->compositor_idle_id = 0;
     display_info->compositor_timeout_id = 0;
 
-    display_info->enable_compositor = ((display_info->have_composite)
+    display_info->enable_compositor = ((display_info->have_render)
+                                    && (display_info->have_composite)
                                     && (display_info->have_damage)
                                     && (display_info->have_fixes));
     if (!display_info->enable_compositor)
@@ -2221,6 +2456,7 @@ compositorManageScreen (ScreenInfo *screen_info, gboolean manual_redirect)
 
     display_info = screen_info->display_info;
     screen_info->compositor_active = FALSE;
+    screen_info->manual_redirect = manual_redirect;
 
     if (!(display_info->enable_compositor))
     {
@@ -2228,7 +2464,7 @@ compositorManageScreen (ScreenInfo *screen_info, gboolean manual_redirect)
     }
 
     gdk_error_trap_push ();
-    if (manual_redirect)
+    if (screen_info->manual_redirect)
     {
         XCompositeRedirectSubwindows (display_info->dpy, screen_info->xroot, CompositeRedirectManual);
     }
@@ -2348,6 +2584,15 @@ compositorUnmanageScreen (ScreenInfo *screen_info)
     }
 
     screen_info->gsize = -1;
+
+    if (screen_info->manual_redirect)
+    {
+        XCompositeUnredirectSubwindows (display_info->dpy, screen_info->xroot, CompositeRedirectManual);
+    }
+    else
+    {
+        XCompositeUnredirectSubwindows (display_info->dpy, screen_info->xroot, CompositeRedirectAutomatic);
+    }
 #endif /* HAVE_COMPOSITOR */
 }
 
@@ -2375,6 +2620,56 @@ compositorRepairScreen (ScreenInfo *screen_info)
 #endif /* HAVE_COMPOSITOR */
 }
 
+void
+compositorUpdateScreenSize (ScreenInfo *screen_info)
+{
+#ifdef HAVE_COMPOSITOR
+    DisplayInfo *display_info = screen_info->display_info;
+    
+    g_return_if_fail (screen_info != NULL);
+    TRACE ("entering compositorUpdateScreenSize");
+
+    display_info = screen_info->display_info;
+    if (screen_info->rootBuffer)
+    {
+        XRenderFreePicture (display_info->dpy, screen_info->rootBuffer);
+        screen_info->rootBuffer = None;
+    }
+    add_repair (display_info);
+#endif /* HAVE_COMPOSITOR */
+}
+
+void
+compositorRebuildScreen (ScreenInfo *screen_info)
+{
+#ifdef HAVE_COMPOSITOR
+    DisplayInfo *display_info = NULL;
+    GList *index;
+
+    g_return_if_fail (screen_info != NULL);
+    TRACE ("entering compositorRepairScreen");
+
+    display_info = screen_info->display_info;
+    if (!(display_info->enable_compositor))
+    {
+        return;
+    }
+
+    if (!(screen_info->compositor_active))
+    {
+        return;
+    }
+
+    for (index = screen_info->cwindows; index; index = g_list_next (index))
+    {
+        CWindow *cw2 = (CWindow *) index->data;
+        free_win_data (cw2, FALSE);
+        init_opacity (cw2);
+    }
+    repair_screen (screen_info);
+#endif /* HAVE_COMPOSITOR */
+}
+
 void
 compositorDamageWindow (DisplayInfo *display_info, Window id)
 {
@@ -2400,27 +2695,30 @@ compositorDamageWindow (DisplayInfo *display_info, Window id)
 #endif /* HAVE_COMPOSITOR */
 }
 
-void
-compositorUpdateWindow (DisplayInfo *display_info, Window id, gint new_width, gint new_height, gboolean shape_notify)
+gboolean
+compositorTestServer (DisplayInfo *display_info)
 {
 #ifdef HAVE_COMPOSITOR
-    CWindow *cw;
+    char *vendor;
 
-    g_return_if_fail (display_info != NULL);
-    g_return_if_fail (id != None);
-    TRACE ("entering compositorUpdateWindow: 0x%lx", id);
+    g_return_val_if_fail (display_info != NULL, FALSE);
+    TRACE ("entering compositorTestServer");
 
-    if (!(display_info->enable_compositor))
+    vendor = ServerVendor (display_info->dpy);
+
+    if (vendor && (!strstr ("X.Org", vendor)))
     {
-        TRACE ("compositor disabled");
-        return;
-    }
-
-    cw = find_cwindow_in_display (display_info, id);
-    if (cw)
-    {
-        TRACE ("Resizing Ox%ld to (%i, %i)", cw->id, new_width, new_height);
-        resize_win (cw, cw->attr.x, cw->attr.y, new_width, new_height, cw->attr.border_width, shape_notify);
+        /* 
+           Check the version, X.org 6.8.x has some bugs that makes
+           it not suitable for use with xfwm4 compositor
+         */
+        if ((VendorRelease(display_info->dpy) / 10) <= 68)
+        {
+            return FALSE;
+        }
     }
+    return TRUE;
+#else /* HAVE_COMPOSITOR */
+    return FALSE;
 #endif /* HAVE_COMPOSITOR */
 }
