@@ -1,4 +1,4 @@
-/*      $Id: compositor.h 12206 2004-12-13 21:27:18Z olivier $
+/*      $Id: compositor.h 20669 2006-03-31 21:54:58Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         xcompmgr - (c) 2003 Keith Packard
-        xfwm4    - (c) 2004 Olivier Fourdan
+        xfwm4    - (c) 2005-2006 Olivier Fourdan
  
  */
 
@@ -45,9 +45,11 @@ void     compositorRepairDisplay    (DisplayInfo *);
 gboolean compositorManageScreen     (ScreenInfo *, gboolean);
 void     compositorUnmanageScreen   (ScreenInfo *);
 void     compositorRepairScreen     (ScreenInfo *);
+void     compositorRebuildScreen    (ScreenInfo *);
+void     compositorUpdateScreenSize (ScreenInfo *);
 
 void     compositorWindowSetOpacity (DisplayInfo *, Window, guint);
 void     compositorDamageWindow     (DisplayInfo *, Window);
-void     compositorUpdateWindow     (DisplayInfo *, Window, gint, gint, gboolean);
+gboolean compositorTestServer       (DisplayInfo *);
 
 #endif /* INC_COMPOSITOR_H */
