@@ -1,4 +1,4 @@
-/*      $Id: frame.c,v 1.76 2004/12/08 21:01:57 olivier Exp $
+/*      $Id: frame.c 19935 2006-02-18 08:21:05Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -192,38 +192,7 @@ frameHeight (Client * c)
 }
 
 static void
-fillRectangle (Display *dpy, int screen, Drawable d, Pixmap pm, int x, int y, int width, int height)
-{
-    XGCValues gv;
-    GC gc;
-    unsigned long mask;
-
-    TRACE ("entering fillRectangle");
-
-    if ((width < 1) || (height < 1))
-    {
-        return;
-    }
-    gv.fill_style = FillTiled;
-    gv.tile = pm;
-    gv.ts_x_origin = x;
-    gv.ts_y_origin = y;
-    gv.foreground = WhitePixel (dpy, screen);
-    if (gv.tile != None)
-    {
-        mask = GCTile | GCFillStyle | GCTileStipXOrigin;
-    }
-    else
-    {
-        mask = GCForeground;
-    }
-    gc = XCreateGC (dpy, d, mask, &gv);
-    XFillRectangle (dpy, d, gc, x, y, width, height);
-    XFreeGC (dpy, gc);
-}
-
-static void
-frameCreateTitlePixmap (Client * c, int state, int left, int right, XfwmPixmap * pm)
+frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap * pm)
 {
     ScreenInfo *screen_info = NULL;
     DisplayInfo *display_info = NULL;
@@ -349,17 +318,11 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, XfwmPixmap *
 
     if (w1 > 0)
     {
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->pixmap, screen_info->title[TITLE_1][state].pixmap,
-            0, 0, w1, frameTop (c));
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->mask, screen_info->title[TITLE_1][state].mask, 0, 0,
-            w1, frameTop (c));
+        xfwmPixmapFill (&screen_info->title[TITLE_1][state], pm, 0, 0, w1, frameTop (c)); 
         x = x + w1;
     }
 
-    fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->pixmap, screen_info->title[TITLE_2][state].pixmap, x, 0,
-        w2, frameTop (c));
-    fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->mask, screen_info->title[TITLE_2][state].mask, x, 0, w2,
-        frameTop (c));
+    xfwmPixmapFill (&screen_info->title[TITLE_2][state], pm, x, 0, w2, frameTop (c)); 
     x = x + w2;
 
     if (w3 > 0)
@@ -372,13 +335,10 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, XfwmPixmap *
         {
             voffset = screen_info->params->title_vertical_offset_inactive;
         }
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->pixmap, screen_info->title[TITLE_3][state].pixmap,
-            x, 0, w3, frameTop (c));
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->mask, screen_info->title[TITLE_3][state].mask, x, 0,
-            w3, frameTop (c));
+        xfwmPixmapFill (&screen_info->title[TITLE_3][state], pm, x, 0, w3, frameTop (c)); 
         if (screen_info->params->title_shadow[state])
         {
-            gdk_gc_get_values (screen_info->black_gc, &values);
+            gdk_gc_get_values (screen_info->title_shadow_colors[state].gc, &values);
             gdk_gc_set_values (gc, &values, GDK_GC_FOREGROUND);
             gdk_draw_layout (gpixmap, gc, x + tp + 1,
                 (frameTop (c) + voffset - logical_rect.height) / 2 + 1,
@@ -395,18 +355,12 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, XfwmPixmap *
     {
         x = right - w4;
     }
-    fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->pixmap, screen_info->title[TITLE_4][state].pixmap, x, 0,
-        w4, frameTop (c));
-    fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->mask, screen_info->title[TITLE_4][state].mask, x, 0, w4,
-        frameTop (c));
+    xfwmPixmapFill (&screen_info->title[TITLE_4][state], pm, x, 0, w4, frameTop (c)); 
     x = x + w4;
 
     if (w5 > 0)
     {
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->pixmap, screen_info->title[TITLE_5][state].pixmap,
-            x, 0, w5, frameTop (c));
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, pm->mask, screen_info->title[TITLE_5][state].mask, x, 0,
-            w5, frameTop (c));
+        xfwmPixmapFill (&screen_info->title[TITLE_5][state], pm, x, 0, w5, frameTop (c)); 
     }
     g_object_unref (G_OBJECT (gc));
     g_object_unref (G_OBJECT (gpixmap));
@@ -512,13 +466,19 @@ getLetterFromButton (int i, Client * c)
     return chr;
 }
 
-static XfwmPixmap *
+static xfwmPixmap *
 frameGetPixmap (Client * c, int button, int state)
 {
     ScreenInfo *screen_info = c->screen_info;
 
     switch (button)
     {
+        case MENU_BUTTON:
+            if (screen_info->params->show_app_icon)
+            {
+                return &c->appmenu[state];
+            }
+            break;
         case SHADE_BUTTON:
             if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED)
                 && screen_info->buttons[SHADE_BUTTON][state + 3].pixmap)
@@ -557,18 +517,18 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
     DisplayInfo *display_info = screen_info->display_info;
     Window temp;
     XRectangle rect;
-    XfwmPixmap *my_pixmap;
+    xfwmPixmap *my_pixmap;
     int i;
    
     TRACE ("entering frameSetShape");
     TRACE ("setting shape for client (0x%lx)", c->window);
 
-    if (!display_info->shape)
+    if (!display_info->have_shape)
     {
         return;
     }
 
-    temp = XCreateSimpleWindow (display_info->dpy, screen_info->xroot, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
+    temp = XCreateSimpleWindow (display_info->dpy, c->frame, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
 
     if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
     {
@@ -744,7 +704,7 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
             if (xfwmWindowVisible (&c->buttons[i]))
             {
                 XShapeCombineShape (display_info->dpy, temp, ShapeBounding, button_x[i],
-                                    (frameTop (c) - screen_info->buttons[i][state].height) / 2,
+                                    (frameTop (c) - screen_info->buttons[i][state].height + 1) / 2,
                                     MYWINDOW_XWINDOW (c->buttons[i]), ShapeBounding, ShapeUnion);
             }
         }
@@ -759,19 +719,14 @@ void
 frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
 {
     ScreenInfo *screen_info = NULL;
-    XfwmPixmap *my_pixmap;
+    xfwmPixmap *my_pixmap;
     gboolean requires_clearing = FALSE;
+    gboolean width_changed = FALSE;
+    gboolean height_changed = FALSE;
+    gboolean title_changed = FALSE;
     int state = ACTIVE;
-    int i;
-    int j;
-    int x;
-    int button;
-    int left;
-    int right;
-    int top_width;
-    int bottom_width;
-    int left_height;
-    int right_height;
+    int i, j, x, button, left, right;
+    int top_width, bottom_width, left_height, right_height;
     int button_x[BUTTON_COUNT];
 
     TRACE ("entering frameDraw");
@@ -816,37 +771,48 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
     /* Flag clearance */
     FLAG_UNSET (c->xfwm_flags,  XFWM_FLAG_FIRST_MAP);
 
+    /* Cache mgmt */
+    if (invalidate_cache)
+    {
+        clientClearPixmapCache (c);
+        requires_clearing = TRUE;
+    }
+    else
+    {
+        if (c->pm_cache.previous_width != c->width)
+        {
+            xfwmPixmapFree (&c->pm_cache.pm_title[ACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_title[INACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][ACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][INACTIVE]);
+            c->pm_cache.previous_width = c->width;
+        }
+        if (c->pm_cache.previous_height != c->height)
+        {
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][ACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][INACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][ACTIVE]);
+            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][INACTIVE]);
+            c->pm_cache.previous_height = c->height;
+        }
+    }
+
+    /* Corners have no caching and get never resized, we need to update them separately */
+    if (requires_clearing)
+    {
+        xfwmWindowSetBG (&c->corners[CORNER_TOP_LEFT], 
+                         &screen_info->corners[CORNER_TOP_LEFT][state]);
+        xfwmWindowSetBG (&c->corners[CORNER_TOP_RIGHT], 
+                         &screen_info->corners[CORNER_TOP_RIGHT][state]);
+        xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_LEFT], 
+                         &screen_info->corners[CORNER_BOTTOM_LEFT][state]);
+        xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_RIGHT], 
+                         &screen_info->corners[CORNER_BOTTOM_RIGHT][state]);
+    }
+
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
-        /* Cache mgmt */
-        if (invalidate_cache)
-        {
-            clientClearPixmapCache (c);
-            requires_clearing = TRUE;
-        }
-        else
-        {
-            if (c->pm_cache.previous_width != c->width)
-            {
-                xfwmPixmapFree (&c->pm_cache.pm_title[ACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_title[INACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][ACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][INACTIVE]);
-                c->pm_cache.previous_width = c->width;
-                requires_clearing = TRUE;
-            }
-            if (c->pm_cache.previous_height != c->height)
-            {
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][ACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][INACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][ACTIVE]);
-                xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][INACTIVE]);
-                c->pm_cache.previous_height = c->height;
-                requires_clearing = TRUE;
-            }
-        }
-
         /* First, hide the buttons that we don't have... */
         for (i = 0; i < BUTTON_COUNT; i++)
         {
@@ -874,11 +840,10 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
                     my_pixmap = frameGetPixmap (c, button, c->button_pressed[button] ? PRESSED : state);
                     if (my_pixmap->pixmap)
                     {
-                        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-                            MYWINDOW_XWINDOW (c->buttons[button]), my_pixmap->pixmap);
+                        xfwmWindowSetBG (&c->buttons[button], my_pixmap);
                     }
                     xfwmWindowShow (&c->buttons[button], x,
-                        (frameTop (c) - screen_info->buttons[button][state].height) / 2,
+                        (frameTop (c) - screen_info->buttons[button][state].height + 1) / 2,
                         screen_info->buttons[button][state].width,
                         screen_info->buttons[button][state].height, TRUE);
                     button_x[button] = x;
@@ -910,13 +875,12 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
                     my_pixmap = frameGetPixmap (c, button, c->button_pressed[button] ? PRESSED : state);
                     if (my_pixmap->pixmap)
                     {
-                        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-                            MYWINDOW_XWINDOW (c->buttons[button]), my_pixmap->pixmap);
+                        xfwmWindowSetBG (&c->buttons[button], my_pixmap);
                     }
                     x = x - screen_info->buttons[button][state].width -
                         screen_info->params->button_spacing;
                     xfwmWindowShow (&c->buttons[button], x,
-                        (frameTop (c) - screen_info->buttons[button][state].height) / 2,
+                        (frameTop (c) - screen_info->buttons[button][state].height + 1) / 2,
                         screen_info->buttons[button][state].width,
                         screen_info->buttons[button][state].height, TRUE);
                     button_x[button] = x;
@@ -948,71 +912,56 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         {
             frameCreateTitlePixmap (c, state, left, right,
                 &c->pm_cache.pm_title[state]);
-            requires_clearing = TRUE;
+            title_changed = TRUE;
         }
 
         if (c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap == None)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_LEFT][state],
                 frameLeft (c), left_height);
-            requires_clearing = TRUE;
+            height_changed = TRUE;
         }
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap,
-            screen_info->sides[SIDE_LEFT][state].pixmap, 0, 0, frameLeft (c),
-            left_height);
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_LEFT][state].mask,
-            screen_info->sides[SIDE_LEFT][state].mask, 0, 0, frameLeft (c),
-            left_height);
-
+        xfwmPixmapFill (&screen_info->sides[SIDE_LEFT][state], 
+                        &c->pm_cache.pm_sides[SIDE_LEFT][state],
+                        0, 0, frameLeft (c), left_height); 
         if (c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap == None)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_RIGHT][state],
                 frameRight (c), right_height);
-            requires_clearing = TRUE;
+            height_changed = TRUE;
         }
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap,
-            screen_info->sides[SIDE_RIGHT][state].pixmap, 0, 0, frameRight (c),
-            right_height);
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_RIGHT][state].mask,
-            screen_info->sides[SIDE_RIGHT][state].mask, 0, 0, frameRight (c),
-            right_height);
-
+        xfwmPixmapFill (&screen_info->sides[SIDE_RIGHT][state], 
+                        &c->pm_cache.pm_sides[SIDE_RIGHT][state], 
+                        0, 0, frameRight (c), right_height); 
         if (c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap == None)
         {
             xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_BOTTOM][state],
                 bottom_width, frameBottom (c));
-            requires_clearing = TRUE;
+            width_changed = TRUE;
         }
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap,
-            screen_info->sides[SIDE_BOTTOM][state].pixmap, 0, 0, bottom_width,
-            frameBottom (c));
-        fillRectangle (clientGetXDisplay (c), screen_info->screen, c->pm_cache.pm_sides[SIDE_BOTTOM][state].mask,
-            screen_info->sides[SIDE_BOTTOM][state].mask, 0, 0, bottom_width,
-            frameBottom (c));
+        xfwmPixmapFill (&screen_info->sides[SIDE_BOTTOM][state], 
+                        &c->pm_cache.pm_sides[SIDE_BOTTOM][state], 
+                        0, 0, bottom_width, frameBottom (c));
 
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c), MYWINDOW_XWINDOW (c->title),
-            c->pm_cache.pm_title[state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
-            c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
-            c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
-            c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]),
-            screen_info->corners[CORNER_TOP_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
-            screen_info->corners[CORNER_TOP_RIGHT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].pixmap);
-        XSetWindowBackgroundPixmap (clientGetXDisplay (c),
-            MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]),
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].pixmap);
+        if (requires_clearing || title_changed)
+        {
+            xfwmWindowSetBG (&c->title, 
+                             &c->pm_cache.pm_title[state]);
+        }
+
+        if (requires_clearing || width_changed)
+        {
+            xfwmWindowSetBG (&c->sides[SIDE_BOTTOM], 
+                             &c->pm_cache.pm_sides[SIDE_BOTTOM][state]);
+        }
+
+        if (requires_clearing || height_changed)
+        {
+            xfwmWindowSetBG (&c->sides[SIDE_LEFT], 
+                             &c->pm_cache.pm_sides[SIDE_LEFT][state]);
+            xfwmWindowSetBG (&c->sides[SIDE_RIGHT], 
+                             &c->pm_cache.pm_sides[SIDE_RIGHT][state]);
+        }
 
         if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
@@ -1022,20 +971,19 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         else
         {
             xfwmWindowShow (&c->sides[SIDE_LEFT], 0, frameTop (c),
-                frameLeft (c), left_height, requires_clearing);
+                frameLeft (c), left_height, (requires_clearing | height_changed));
             xfwmWindowShow (&c->sides[SIDE_RIGHT],
                 frameWidth (c) - frameRight (c), frameTop (c), frameRight (c),
-                right_height, requires_clearing);
+                right_height, (requires_clearing | height_changed));
         }
 
         xfwmWindowShow (&c->title,
             screen_info->corners[CORNER_TOP_LEFT][state].width, 0, top_width,
-            frameTop (c), requires_clearing);
+            frameTop (c), (requires_clearing | title_changed));
         xfwmWindowShow (&c->sides[SIDE_BOTTOM],
             screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
             frameHeight (c) - frameBottom (c), bottom_width, frameBottom (c),
-            requires_clearing);
-
+            (requires_clearing | width_changed));
         xfwmWindowShow (&c->corners[CORNER_TOP_LEFT], 0, 0,
             screen_info->corners[CORNER_TOP_LEFT][state].width,
             screen_info->corners[CORNER_TOP_LEFT][state].height,
@@ -1067,24 +1015,27 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
     }
     else
     {
-        xfwmWindowHide (&c->title);
+        if (xfwmWindowVisible (&c->title))
+        {
+            xfwmWindowHide (&c->title);
+        }
         for (i = 0; i < 3; i++)
         {
-            if (MYWINDOW_XWINDOW (c->sides[i]))
+            if (MYWINDOW_XWINDOW (c->sides[i]) && xfwmWindowVisible (&c->sides[i]))
             {
                 xfwmWindowHide (&c->sides[i]);
             }
         }
         for (i = 0; i < 4; i++)
         {
-            if (MYWINDOW_XWINDOW (c->corners[i]))
+            if (MYWINDOW_XWINDOW (c->corners[i]) && xfwmWindowVisible (&c->corners[i]))
             {
                 xfwmWindowHide (&c->corners[i]);
             }
         }
         for (i = 0; i < BUTTON_COUNT; i++)
         {
-            if (MYWINDOW_XWINDOW (c->buttons[i]))
+            if (MYWINDOW_XWINDOW (c->buttons[i]) && xfwmWindowVisible (&c->buttons[i]))
             {
                 xfwmWindowHide (&c->buttons[i]);
             }
