@@ -1,4 +1,4 @@
-/*      $Id: menu.h 12124 2004-11-01 15:50:24Z olivier $
+/*      $Id: menu.h 20527 2006-03-24 12:26:41Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -95,7 +95,7 @@ Menu *menu_default             (GdkScreen *,
                                 MenuFunc, 
                                 gint, 
                                 gint, 
-                                gchar *, 
+                                gchar **, 
                                 gint, 
                                 XfceFilterSetup*, 
                                 gpointer);
