@@ -1,8 +1,9 @@
-/*
+/*      $Id: parserc.c 19935 2006-02-18 08:21:05Z olivier $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -23,22 +24,23 @@
 #endif
 
 #include <X11/Xlib.h>
-#include <libxfce4util/debug.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
-#include "parserc.h"
+#include <libxfce4util/libxfce4util.h> 
+
 #include "settings.h"
+#include "parserc.h"
 
 #ifndef DEFAULT_THEME
-#define DEFAULT_THEME "default"
+#define DEFAULT_THEME "Default"
 #endif
 
 gboolean
-parseRc (const gchar * file, const gchar * dir, Settings rc[])
+parseRc (const gchar * file, const gchar * dir, Settings *rc)
 {
-    gchar buf[80];
+    gchar buf[255];
     gchar *filename, *lvalue, *rvalue;
     FILE *fp;
 
@@ -48,7 +50,7 @@ parseRc (const gchar * file, const gchar * dir, Settings rc[])
 
     if (dir)
     {
-        filename = g_build_filename (dir, G_DIR_SEPARATOR_S, file, NULL);
+        filename = g_build_filename (dir, file, NULL);
     }
     else
     {
@@ -76,7 +78,7 @@ parseRc (const gchar * file, const gchar * dir, Settings rc[])
 }
 
 gboolean
-checkRc (Settings rc[])
+checkRc (Settings *rc)
 {
     gint i;
     gboolean rval = TRUE;
@@ -95,7 +97,7 @@ checkRc (Settings rc[])
 }
 
 gchar *
-getValue (const gchar * option, Settings rc[])
+getValue (const gchar * option, Settings *rc)
 {
     gint i;
 
@@ -114,7 +116,7 @@ getValue (const gchar * option, Settings rc[])
 }
 
 gboolean
-setValue (const gchar * lvalue, const gchar * rvalue, Settings rc[])
+setValue (const gchar * lvalue, const gchar * rvalue, Settings *rc)
 {
     gint i;
 
@@ -143,13 +145,13 @@ setValue (const gchar * lvalue, const gchar * rvalue, Settings rc[])
 }
 
 gboolean
-setBooleanValueFromInt (const gchar * lvalue, int value, Settings rc[])
+setBooleanValueFromInt (const gchar * lvalue, int value, Settings *rc)
 {
     return setValue (lvalue, value ? "true" : "false", rc);
 }
 
 gboolean
-setIntValueFromInt (const gchar * lvalue, int value, Settings rc[])
+setIntValueFromInt (const gchar * lvalue, int value, Settings *rc)
 {
     gchar *s;
     gboolean result;
@@ -162,12 +164,19 @@ setIntValueFromInt (const gchar * lvalue, int value, Settings rc[])
 }
 
 gchar *
-getThemeDir (const gchar * theme)
+
+getSystemThemeDir (void)
+{
+    return g_build_filename (DATADIR, "themes", DEFAULT_THEME, "xfwm4", NULL);
+}
+
+gchar *
+getThemeDir (const gchar * theme, const gchar * file)
 {
     if (!theme)
     {
-        return g_build_filename (DATADIR, G_DIR_SEPARATOR_S, "themes",
-            G_DIR_SEPARATOR_S, DEFAULT_THEME, NULL);
+        return g_build_filename (DATADIR, "themes", DEFAULT_THEME, "xfwm4", 
+                                 NULL);
     }
     else if (g_path_is_absolute (theme))
     {
@@ -177,37 +186,36 @@ getThemeDir (const gchar * theme)
         }
         else
         {
-            return g_build_filename (DATADIR, G_DIR_SEPARATOR_S, "themes",
-                G_DIR_SEPARATOR_S, DEFAULT_THEME, NULL);
+            return getSystemThemeDir ();
         }
     }
     else
     {
-        gchar *test = g_build_filename (g_get_home_dir (), G_DIR_SEPARATOR_S,
-            ".themes",
-            G_DIR_SEPARATOR_S, "xfwm4",
-            G_DIR_SEPARATOR_S,
-            theme, NULL);
-        if (g_file_test (test, G_FILE_TEST_IS_DIR))
+        gchar *test_file, *path;
+        
+        path = g_build_filename (theme, "xfwm4", file, NULL);
+
+        xfce_resource_push_path (XFCE_RESOURCE_THEMES, 
+                                 DATADIR G_DIR_SEPARATOR_S "themes");
+        test_file = xfce_resource_lookup (XFCE_RESOURCE_THEMES, path);
+        xfce_resource_pop_path (XFCE_RESOURCE_THEMES);
+
+        g_free (path);
+ 
+        if (test_file)
         {
-            return test;
+            path = g_path_get_dirname (test_file);
+            g_free (test_file);
+            return path;
         }
-        g_free (test);
-        test =
-            g_build_filename (DATADIR, G_DIR_SEPARATOR_S, "themes",
-            G_DIR_SEPARATOR_S, theme, NULL);
-        if (g_file_test (test, G_FILE_TEST_IS_DIR))
-        {
-            return test;
-        }
-        g_free (test);
     }
-    return g_build_filename (DATADIR, G_DIR_SEPARATOR_S, "themes",
-        G_DIR_SEPARATOR_S, DEFAULT_THEME, NULL);
+    
+    /* Pfew, really can't find that theme nowhere! */
+    return getSystemThemeDir ();
 }
 
 void
-freeRc (Settings rc[])
+freeRc (Settings *rc)
 {
     gint i;
 
