@@ -1,4 +1,4 @@
-/*      $Id: poswin.c 11914 2004-08-22 22:18:50Z olivier $
+/*      $Id: poswin.c 19935 2006-02-18 08:21:05Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
           based on a patch from Joshua Blanton <jblanton@irg.cs.ohiou.edu>
  */
 
@@ -122,7 +122,7 @@ poswinShow (Poswin * poswin)
 {
     g_return_if_fail (poswin != NULL);
     
-    gtk_widget_show_now (poswin->window);
+    gtk_widget_show (poswin->window);
 }
 
 void
