@@ -1,4 +1,4 @@
-/*      $Id: screen.c,v 1.12 2004/12/30 17:58:30 olivier Exp $
+/*      $Id: screen.c 20527 2006-03-24 12:26:41Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -28,8 +28,13 @@
 #include <X11/extensions/shape.h>
 #include <gtk/gtk.h>
 #include <glib.h>
+#include <libxfce4util/libxfce4util.h> 
 #include <libxfcegui4/libxfcegui4.h>
 
+#ifdef HAVE_RENDER
+#include <X11/extensions/Xrender.h>
+#endif
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 #define SN_API_NOT_YET_FROZEN
 #include <libsn/sn.h>
@@ -44,7 +49,10 @@
 ScreenInfo *
 myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_mask)
 {
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
     gchar selection[32];
+#endif
+    long desktop_visible = 0;
     ScreenInfo *screen_info = NULL;
     GdkWindow *event_win;
     PangoLayout *layout = NULL;
@@ -52,6 +60,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     
     g_return_val_if_fail (display_info, NULL);
     g_return_val_if_fail (GDK_IS_SCREEN (gscr), NULL);
+    TRACE ("entering myScreenInit");
 
     screen_info = g_new0 (ScreenInfo, 1);
     screen_info->params = g_new0 (XfwmParams, 1);
@@ -94,6 +103,8 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->visual = DefaultVisual (display_info->dpy, screen_info->screen);
     screen_info->current_ws = 0;
     screen_info->previous_ws = 0;
+    screen_info->current_ws = 0;
+    screen_info->previous_ws = 0;
 
     screen_info->margins[TOP] = screen_info->gnome_margins[TOP] = 0;
     screen_info->margins[LEFT] = screen_info->gnome_margins[LEFT] = 0;
@@ -102,7 +113,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 
     screen_info->workspace_count = -1;
     screen_info->workspace_names = NULL;
-    screen_info->workspace_names_length = 0;
+    screen_info->workspace_names_items = 0;
 
     screen_info->mcs_client = NULL;
     screen_info->mcs_initted = FALSE;
@@ -115,32 +126,57 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->client_serial = 0L;
     screen_info->button_handler_id = 0L;
     
+    screen_info->key_grabs = 0;
+    screen_info->pointer_grabs = 0;
+
+    getHint (display_info, screen_info->xroot, NET_SHOWING_DESKTOP, &desktop_visible);
+    screen_info->show_desktop = (desktop_visible != 0);
+    
     /* Create the side windows to detect edge movement */
-    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[0], 
-                                  0, 0, 
-                                  1, gdk_screen_get_height (gscr), 
-                                  LeaveWindowMask | PointerMotionMask);
 
-    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[1], 
-                                  gdk_screen_get_width (gscr) - 1, 0, 
-                                  1, gdk_screen_get_height (gscr), 
-                                  LeaveWindowMask | PointerMotionMask);
+    /*left*/
+    xfwmWindowTemp (screen_info,
+                    NULL, 0,
+                    screen_info->xroot,
+                    &screen_info->sidewalk[0], 
+                    0, 0, 
+                    1, gdk_screen_get_height (gscr), 
+                    EnterWindowMask);
 
-    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[2], 
-                                  0, 0, 
-                                  gdk_screen_get_width (gscr), 1,
-                                  LeaveWindowMask | PointerMotionMask);
+    /*right*/
+    xfwmWindowTemp (screen_info,
+                    NULL, 0,
+                    screen_info->xroot,
+                    &screen_info->sidewalk[1], 
+                    gdk_screen_get_width (gscr) - 1, 0, 
+                    1, gdk_screen_get_height (gscr), 
+                    EnterWindowMask);
 
-    xfwmWindowTemp (display_info->dpy, screen_info->xroot, &screen_info->sidewalk[3], 
-                                  0, gdk_screen_get_height (gscr) - 1, 
-                                  gdk_screen_get_width (gscr), 1,
-                                  LeaveWindowMask | PointerMotionMask);
+    /*top*/
+    xfwmWindowTemp (screen_info,
+                    NULL, 0,
+                    screen_info->xroot,
+                    &screen_info->sidewalk[2], 
+                    0, 0, 
+                    gdk_screen_get_width (gscr), 1,
+                    EnterWindowMask);
 
-    screen_info->gnome_win = GDK_WINDOW_XWINDOW (screen_info->gtk_win->window);
+    /*bottom*/
+    xfwmWindowTemp (screen_info,
+                    NULL, 0,
+                    screen_info->xroot,
+                    &screen_info->sidewalk[3], 
+                    0, gdk_screen_get_height (gscr) - 1, 
+                    gdk_screen_get_width (gscr), 1,
+                    EnterWindowMask);
 
+    screen_info->xfwm4_win = GDK_WINDOW_XWINDOW (screen_info->gtk_win->window);
+
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
     g_snprintf (selection, sizeof (selection), "_NET_SYSTEM_TRAY_S%d", screen_info->screen);
     screen_info->net_system_tray_selection = XInternAtom (display_info->dpy, selection, FALSE);
     screen_info->systray = getSystrayWindow (display_info, screen_info->net_system_tray_selection);
+#endif
     
     screen_info->box_gc = None;
     screen_info->black_gc = NULL;
@@ -149,6 +185,10 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->title_colors[ACTIVE].allocated = FALSE;
     screen_info->title_colors[INACTIVE].gc = NULL;
     screen_info->title_colors[INACTIVE].allocated = FALSE;
+    screen_info->title_shadow_colors[ACTIVE].gc = NULL;
+    screen_info->title_shadow_colors[ACTIVE].allocated = FALSE;
+    screen_info->title_shadow_colors[INACTIVE].gc = NULL;
+    screen_info->title_shadow_colors[INACTIVE].allocated = FALSE;
 
     for (i = 0; i < BUTTON_COUNT; i++)
     {
@@ -184,16 +224,20 @@ myScreenClose (ScreenInfo *screen_info)
     DisplayInfo *display_info = NULL;
     
     g_return_val_if_fail (screen_info, NULL);
-    
+    TRACE ("entering myScreenClose");
+
     display_info = screen_info->display_info;
     
     clientUnframeAll (screen_info);
     compositorUnmanageScreen (screen_info);
     closeSettings (screen_info);
 
-    g_free (screen_info->workspace_names);
+    if (screen_info->workspace_names)
+    {
+        g_strfreev (screen_info->workspace_names);
+    }
     screen_info->workspace_names = NULL;
-    screen_info->workspace_names_length = 0;
+    screen_info->workspace_names_items = 0;
 
     xfwmWindowDelete (&screen_info->sidewalk[0]);
     xfwmWindowDelete (&screen_info->sidewalk[1]);
@@ -223,7 +267,8 @@ myScreenGetXDisplay (ScreenInfo *screen_info)
     
     g_return_val_if_fail (screen_info, NULL);
     g_return_val_if_fail (screen_info->display_info, NULL);
-    
+    TRACE ("entering myScreenGetXDisplay");
+
     display_info = screen_info->display_info;
     return display_info->dpy;
 }
@@ -232,7 +277,8 @@ GtkWidget *
 myScreenGetGtkWidget (ScreenInfo *screen_info)
 {
     g_return_val_if_fail (screen_info, NULL);
-    
+    TRACE ("entering myScreenGetGtkWidget");
+
     return screen_info->gtk_win;
 }
 
@@ -240,6 +286,123 @@ GdkWindow *
 myScreenGetGdkWindow (ScreenInfo *screen_info)
 {
     g_return_val_if_fail (screen_info, NULL);
-    
+    TRACE ("entering myScreenGetGdkWindow");
+
     return screen_info->gtk_win->window;
 }
+
+gboolean
+myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
+{
+    gboolean grab = TRUE;
+
+    g_return_val_if_fail (screen_info, FALSE);
+    TRACE ("entering myScreenGrabKeyboard");
+
+    if (screen_info->key_grabs == 0)
+    {
+        grab = (XGrabKeyboard (myScreenGetXDisplay (screen_info), 
+                               screen_info->xfwm4_win,
+                               FALSE, 
+                               GrabModeAsync, GrabModeAsync, 
+                               time) == GrabSuccess);
+    }
+    screen_info->key_grabs++;
+    TRACE ("global key grabs %i", screen_info->key_grabs);
+    
+    return grab;
+}
+
+gboolean
+myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, Time time)
+{
+    gboolean grab = TRUE;
+
+    g_return_val_if_fail (screen_info, FALSE);
+    TRACE ("entering myScreenGrabPointer");
+
+    if (screen_info->pointer_grabs == 0)
+    {
+        grab = (XGrabPointer (myScreenGetXDisplay (screen_info), 
+                              screen_info->xfwm4_win, 
+                              FALSE, event_mask, 
+                              GrabModeAsync, GrabModeAsync, 
+                              screen_info->xroot, 
+                              cursor, 
+                              time) == GrabSuccess);
+    }
+    screen_info->pointer_grabs++;
+    TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
+    
+    return grab;
+}
+
+unsigned int
+myScreenUngrabKeyboard (ScreenInfo *screen_info, Time time)
+{
+    g_return_val_if_fail (screen_info, 0);
+    TRACE ("entering myScreenUngrabKeyboard");
+
+    screen_info->key_grabs = screen_info->key_grabs - 1;
+    if (screen_info->key_grabs < 0)
+    {
+        screen_info->key_grabs = 0;
+    }
+    if (screen_info->key_grabs == 0)
+    {
+        XUngrabKeyboard (myScreenGetXDisplay (screen_info), time);
+    }
+    TRACE ("global key grabs %i", screen_info->key_grabs);
+
+    return screen_info->key_grabs;
+}
+
+unsigned int
+myScreenUngrabPointer (ScreenInfo *screen_info, Time time)
+{
+    g_return_val_if_fail (screen_info, 0);
+    TRACE ("entering myScreenUngrabPointer");
+
+    screen_info->pointer_grabs = screen_info->pointer_grabs - 1;
+    if (screen_info->pointer_grabs < 0)
+    {
+        screen_info->pointer_grabs = 0;
+    }
+    if (screen_info->pointer_grabs == 0)
+    {
+        XUngrabPointer (myScreenGetXDisplay (screen_info), time);
+    }
+    TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
+
+    return screen_info->pointer_grabs;
+}
+
+void
+myScreenGrabKeys (ScreenInfo *screen_info)
+{
+    Display *dpy;
+
+    dpy = myScreenGetXDisplay (screen_info);
+
+    ungrabKeys (dpy, screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_CYCLE_WINDOWS], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_NEXT_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_PREV_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_ADD_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_DEL_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_1], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_2], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_3], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_4], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_5], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_6], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_7], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_8], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_WORKSPACE_9], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_UP_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_DOWN_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_LEFT_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_RIGHT_WORKSPACE], screen_info->xroot);
+    grabKey (dpy, &screen_info->params->keys[KEY_SHOW_DESKTOP], screen_info->xroot);
+}
+
