@@ -1,4 +1,4 @@
-/*      $Id: screen.h 12370 2005-02-12 21:29:37Z olivier $
+/*      $Id: screen.h 20527 2006-03-24 12:26:41Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -58,9 +58,6 @@ struct _ScreenInfo
     /* The display this screen belongs to */
     DisplayInfo *display_info;
     
-    /* There can be one systray per screen */
-    Atom net_system_tray_selection;
-    
     /* Window stacking, per screen */
     GList *windows_stack;
     Client *last_raise;
@@ -73,10 +70,11 @@ struct _ScreenInfo
     
     /* Theme pixmaps and other params, per screen */
     XfwmColor title_colors[2];
-    XfwmPixmap buttons[BUTTON_COUNT][6];
-    XfwmPixmap corners[4][2];
-    XfwmPixmap sides[3][2];
-    XfwmPixmap title[5][2];
+    XfwmColor title_shadow_colors[2];
+    xfwmPixmap buttons[BUTTON_COUNT][6];
+    xfwmPixmap corners[4][2];
+    xfwmPixmap sides[3][2];
+    xfwmPixmap title[5][2];
 
     /* Per screen graphic contexts */
     GC box_gc;
@@ -92,9 +90,8 @@ struct _ScreenInfo
 
     GtkWidget *gtk_win;
     xfwmWindow sidewalk[4];
-    Window gnome_win;
+    Window xfwm4_win;
     Window xroot;
-    Window systray;
     
     int gnome_margins[4];
     int margins[4];
@@ -104,8 +101,8 @@ struct _ScreenInfo
 
     /* Workspace definitions */
     int workspace_count;
-    gchar *workspace_names;
-    int workspace_names_length;
+    gchar **workspace_names;
+    int workspace_names_items;
     NetWmDesktopLayout desktop_layout;
 
     /* Button handler for GTK */
@@ -118,6 +115,15 @@ struct _ScreenInfo
     /* Per screen parameters */
     XfwmParams *params;
 
+    /* show desktop flag */
+    gboolean show_desktop;
+    
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
+    /* There can be one systray per screen */
+    Atom net_system_tray_selection;
+    Window systray;
+#endif
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     /* Startup notification data, per screen */
     SnMonitorContext *sn_context;
@@ -127,7 +133,8 @@ struct _ScreenInfo
 
 #ifdef HAVE_COMPOSITOR
     gboolean compositor_active;
-
+    gboolean manual_redirect;
+    
     GList *cwindows;
 
     gaussian_conv *gaussianMap;
@@ -163,4 +170,5 @@ unsigned int     myScreenUngrabKeyboard (ScreenInfo *,
                                          Time);
 unsigned int     myScreenUngrabPointer  (ScreenInfo *, 
                                          Time);
+void             myScreenGrabKeys       (ScreenInfo *);
 #endif /* INC_SCREEN_H */
