@@ -1,4 +1,4 @@
-/*      $Id: stacking.c 18619 2005-11-03 21:58:46Z olivier $
+/*      $Id: stacking.c 19935 2006-02-18 08:21:05Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -252,7 +252,7 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
                 && (c2 != exclude))
             {
                 c = c2;
-        	break;
+                break;
             }
         }
     }
@@ -261,7 +261,7 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
 }
 
 void
-clientRaise (Client * c)
+clientRaise (Client * c, Window wsibling)
 {
     ScreenInfo *screen_info = NULL;
     DisplayInfo *display_info = NULL;
@@ -277,7 +277,7 @@ clientRaise (Client * c)
         TRACE ("client \"%s\" (0x%lx) already raised", c->name, c->window);
         return;
     }
-    TRACE ("raising client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("raising client \"%s\" (0x%lx) over (0x%lx)", c->name, c->window, wsibling);
 
     if (c == clientGetFocus ())
     {
@@ -300,8 +300,32 @@ clientRaise (Client * c)
 
         /* Copy the existing window stack temporarily as reference */
         windows_stack_copy = g_list_copy (screen_info->windows_stack);
-        /* Search for the window that will be just on top of the raised window (layers...) */
-        client_sibling = clientGetNextTopMost (screen_info, c->win_layer, c);
+        /* Search for the window that will be just on top of the raised window  */
+        if (wsibling)
+        {
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            if (c2)
+            {
+                sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
+                if (sibling)
+                {
+                    index1 = g_list_next (sibling);
+                    if (index1)
+                    {
+                        client_sibling = (Client *) index1->data;
+                        /* Do not place window under higher layers though */
+                        if ((client_sibling) && (client_sibling->win_layer < c->win_layer))
+                        {
+                            client_sibling = NULL;
+                        }
+                    }
+                }
+            }
+        }
+        if (!client_sibling)
+        {
+            client_sibling = clientGetNextTopMost (screen_info, c->win_layer, c);
+        }
         screen_info->windows_stack = g_list_remove (screen_info->windows_stack, (gconstpointer) c);
         if (client_sibling)
         {
@@ -389,14 +413,14 @@ clientRaise (Client * c)
 }
 
 void
-clientLower (Client * c)
+clientLower (Client * c, Window wsibling)
 {
     ScreenInfo *screen_info = NULL;
     DisplayInfo *display_info = NULL;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientLower");
-    TRACE ("lowering client \"%s\" (0x%lx)", c->name, c->window);
+    TRACE ("lowering client \"%s\" (0x%lx) below (0x%lx)", c->name, c->window, wsibling);
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
@@ -409,6 +433,10 @@ clientLower (Client * c)
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
     {
         Client *client_sibling = NULL;
+        GList *sibling = NULL;
+        Client *c2 = NULL;
+        GList *index = NULL;
+        gint position;
 
         if (clientIsTransientOrModalForGroup (c))
         {
@@ -418,6 +446,27 @@ clientLower (Client * c)
         {
             client_sibling = clientGetTransient (c);
         }
+        else if (wsibling)
+        {
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            if (c2)
+            {
+                sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
+                if (sibling)
+                {
+                    index = g_list_previous (sibling);
+                    if (index)
+                    {
+                        client_sibling = (Client *) index->data;
+                        /* Do not place window above lower layers though */
+                        if ((client_sibling) && (client_sibling->win_layer > c->win_layer))
+                        {
+                            client_sibling = NULL;
+                        }
+                    }
+                }
+            }
+        }
         if ((!client_sibling) || 
             (client_sibling && (client_sibling->win_layer < c->win_layer)))
         {
@@ -429,8 +478,8 @@ clientLower (Client * c)
             /* Paranoid check to avoid circular linked list */
             if (client_sibling)
             {
-                GList *sibling = g_list_find (screen_info->windows_stack, (gconstpointer) client_sibling);
-                gint position = g_list_position (screen_info->windows_stack, sibling) + 1;
+                sibling = g_list_find (screen_info->windows_stack, (gconstpointer) client_sibling);
+                position = g_list_position (screen_info->windows_stack, sibling) + 1;
 
                 screen_info->windows_stack = g_list_insert (screen_info->windows_stack, c, position);
                 TRACE ("lowest client is \"%s\" (0x%lx) at position %i",
@@ -522,7 +571,6 @@ clientAddToList (Client * c)
 
     clientSetNetClientList (screen_info, display_info->atoms[NET_CLIENT_LIST], screen_info->windows);
     clientSetNetClientList (screen_info, display_info->atoms[WIN_CLIENT_LIST], screen_info->windows);
-    clientSetNetClientList (screen_info, display_info->atoms[NET_CLIENT_LIST_STACKING], screen_info->windows_stack);
 
     FLAG_SET (c->xfwm_flags, XFWM_FLAG_MANAGED);
 }
