@@ -1,4 +1,4 @@
-/*      $Id: startup_notification.c 16044 2005-06-22 21:56:39Z olivier $
+/*      $Id: startup_notification.c 20586 2006-03-26 22:49:20Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         Metacity - (c) 2003 Havoc Pennington
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -274,7 +274,18 @@ sn_client_startup_properties (Client * c)
     if (sequence != NULL)
     {
         int workspace;
+        Time timestamp;
 
+        /* Set initial time */
+        timestamp = sn_startup_sequence_get_timestamp (sequence);
+        TRACE ("Given startup time: %u", (unsigned int) timestamp);
+        if (timestamp > c->user_time)
+        {
+            c->user_time = timestamp;
+        }
+        FLAG_SET (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME);
+        
+        /* Set initial workspace */
         if (!FLAG_TEST (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET))
         {
             workspace = sn_startup_sequence_get_workspace (sequence);
