@@ -1,4 +1,4 @@
-/*      $Id: workspaces.c 13520 2005-04-10 20:33:31Z olivier $
+/*      $Id: workspaces.c 20527 2006-03-24 12:26:41Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -45,24 +45,24 @@
 static void
 workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 {
-    NetWmDesktopLayout * l = &screen_info->desktop_layout;
+    NetWmDesktopLayout l = screen_info->desktop_layout;
     int major_length, minor_length, tmp;
 
-    if (l->orientation == NET_WM_ORIENTATION_HORZ)
+    if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
-        major_length = l->cols;
-        minor_length = l->rows;
+        major_length = l.cols;
+        minor_length = l.rows;
     }
     else
     {
-        major_length = l->rows;
-        minor_length = l->cols;
+        major_length = l.rows;
+        minor_length = l.cols;
     }
 
     *row = n / major_length;
     *col = n % major_length;
 
-    switch (l->start)
+    switch (l.start)
     {
         case NET_WM_TOPRIGHT:
             *col = major_length - *col - 1;
@@ -78,15 +78,15 @@ workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
             break;
     }
 
-    if (l->orientation == NET_WM_ORIENTATION_VERT)
+    if (l.orientation == NET_WM_ORIENTATION_VERT)
     {
         tmp = *row;
         *row = *col;
         *col = tmp;
-        if ((l->start == NET_WM_TOPRIGHT) || (l->start == NET_WM_BOTTOMLEFT))
+        if ((l.start == NET_WM_TOPRIGHT) || (l.start == NET_WM_BOTTOMLEFT))
         {
-            *row = l->rows - *row - 1;
-            *col = l->cols - *col - 1;
+            *row = l.rows - *row - 1;
+            *col = l.cols - *col - 1;
         }
     }
 }
@@ -94,33 +94,33 @@ workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 static int
 workspaceGetNumber (ScreenInfo *screen_info, int row, int col)
 {
-    NetWmDesktopLayout * l = &screen_info->desktop_layout;
+    NetWmDesktopLayout l = screen_info->desktop_layout;
     int major_length, minor_length, n, tmp;
 
-    if (l->orientation == NET_WM_ORIENTATION_HORZ)
+    if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
-        major_length = l->cols;
-        minor_length = l->rows;
+        major_length = l.cols;
+        minor_length = l.rows;
     }
     else
     {
-        major_length = l->rows;
-        minor_length = l->cols;
+        major_length = l.rows;
+        minor_length = l.cols;
     }
 
-    if (l->orientation == NET_WM_ORIENTATION_VERT)
+    if (l.orientation == NET_WM_ORIENTATION_VERT)
     {
         tmp = row;
         row = col;
         col = tmp;
-        if ((l->start == NET_WM_TOPRIGHT) || (l->start == NET_WM_BOTTOMLEFT))
+        if ((l.start == NET_WM_TOPRIGHT) || (l.start == NET_WM_BOTTOMLEFT))
         {
             row = minor_length - row - 1;
             col = major_length - col - 1;
         }
     }
 
-    switch (l->start)
+    switch (l.start)
     {
         case NET_WM_TOPRIGHT:
             col = major_length - col - 1;
@@ -136,7 +136,7 @@ workspaceGetNumber (ScreenInfo *screen_info, int row, int col)
             break;
     }
 
-    n = row*major_length + col;
+    n = (row * major_length) + col;
     return n;
 }
 
@@ -171,7 +171,6 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
 }
 
 /* returns TRUE if the workspace was changed, FALSE otherwise */
-
 gboolean
 workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
 {
@@ -190,7 +189,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
     previous_ws = screen_info->current_ws;
     if ((n >= 0) && (n < screen_info->workspace_count))
     {
-        workspaceSwitch(screen_info, n, c2);
+        workspaceSwitch(screen_info, n, c2, TRUE);
     }
     else if (screen_info->params->wrap_layout)
     {
@@ -219,14 +218,14 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
 
             n = workspaceGetNumber(screen_info, newrow, newcol);
         }
-        workspaceSwitch(screen_info, n, c2);
+        workspaceSwitch(screen_info, n, c2, TRUE);
     }
 
     return (screen_info->current_ws != previous_ws);
 }
 
 void
-workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
+workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus)
 {
     DisplayInfo *display_info = screen_info->display_info;
     Client *c, *new_focus = NULL;
@@ -362,29 +361,22 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
 
     myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
 
-    if (new_focus)
+    if (new_focus && update_focus)
     {
         clientSetFocus (new_focus->screen_info, new_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
     }
-#if 0
-    else
-    {
-        /* If we can't get a window to focus, just pick the one on top */
-        clientFocusTop (screen_info, WIN_LAYER_ABOVE_DOCK);
-    }
-#endif
 }
 
 void
-workspaceSetNames (ScreenInfo * screen_info, char *names, int length)
+workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
 {
     if (screen_info->workspace_names)
     {
-        g_free (screen_info->workspace_names);
+        g_strfreev (screen_info->workspace_names);
     }
 
     screen_info->workspace_names = names;
-    screen_info->workspace_names_length = length;
+    screen_info->workspace_names_items = items;
 }
 
 void
@@ -419,7 +411,7 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
     }
     if (screen_info->current_ws > count - 1)
     {
-        workspaceSwitch (screen_info, count - 1, NULL);
+        workspaceSwitch (screen_info, count - 1, NULL, TRUE);
     }
     setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
                     gdk_screen_get_width (screen_info->gscr),
@@ -479,5 +471,7 @@ workspaceUpdateArea (ScreenInfo *screen_info)
                         gdk_screen_get_width (screen_info->gscr),
                         gdk_screen_get_height (screen_info->gscr),
                         screen_info->margins);
+        /* Also prevent windows from being off screen, just like when screen is resized */
+        clientScreenResize(screen_info);
     }
 }
