@@ -1,4 +1,4 @@
-/*      $Id: workspaces.h 12046 2004-10-08 20:36:07Z olivier $
+/*      $Id: workspaces.h 20527 2006-03-24 12:26:41Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -34,8 +34,8 @@
 #include "client.h"
 
 gboolean workspaceMove (ScreenInfo *, int, int, Client *);
-void workspaceSwitch (ScreenInfo *, int, Client *);
-void workspaceSetNames (ScreenInfo *, char *, int);
+void workspaceSwitch (ScreenInfo *, int, Client *, gboolean);
+void workspaceSetNames (ScreenInfo *, char **, int);
 void workspaceSetCount (ScreenInfo *, int);
 void workspaceUpdateArea (ScreenInfo *);
 
